#version 130

uniform vec3 camPos;
uniform sampler2D normalTex;	//normals
uniform sampler2D depthTex;	//depth
#ifndef EXCLUDE_COLORS
	uniform sampler2D colorTex;	//colors
#endif
uniform mat4 inverseM;
uniform float radius;
uniform float weightModifier;
uniform float drawMode;
uniform float contrast;
uniform float brightness;
uniform float viewportWidth;
uniform float viewportHeight;

void main()
{		
	//ivec2 texDimensions = textureSize(normalTex, 0);
	//float width = 1024;
	//float height = 768;
	//float width = texDimensions.x;
	//float height = texDimensions.y;
	float width = viewportWidth;
	float height = viewportHeight;
	
	//retrieve summed normals from attribute pass and normalize
	vec4 summedNormals = texture2D(normalTex,vec2(gl_FragCoord.x/width, gl_FragCoord.y/height));
	float normalLength = length(summedNormals.xyz);
	if (normalLength == 0.0)
	{
		//gl_FragColor = vec4(1.0,1.0,1.0,1.0);
		gl_FragColor = vec4(0.0,0.0,0.0,1.0);
	}
	else
	{
		#ifdef USE_LIGHTING
			vec3 avgNormal = summedNormals.xyz / normalLength;
			if (avgNormal.z < 0)
				avgNormal = -avgNormal;
			
			//calculate world position
			float depth = float(texture2D(depthTex,vec2(gl_FragCoord.x/width, gl_FragCoord.y/height)));
			vec4 screenPos = vec4(gl_FragCoord.x/width,gl_FragCoord.y/height, depth, 1.0) * 2.0 - 1.0;
			vec4 worldPos = inverseM * screenPos;
			worldPos.x = worldPos.x / worldPos.w;
			worldPos.y = worldPos.y / worldPos.w;
			worldPos.z = worldPos.z / worldPos.w;
		#endif
		
		#ifndef EXCLUDE_COLORS
			vec4 summedColors = texture2D(colorTex,vec2(gl_FragCoord.x/width, gl_FragCoord.y/height));
			//float colorLength = length(summedColors.xyz);
			vec3 avgColor = summedColors.xyz / summedColors.w;
		#endif
		
		#ifdef USE_LIGHTING
			//lighting
			//vec3 lightDir = vec3(-0.316,-0.316,-0.8944);
			//vec3 lightDir = vec3(-0.316, 0.8944, -0.316);
			//vec3 lightDir = vec3(-0.8944,-0.316,-0.316);
			vec3 lightDir = vec3(-1.0,0.0,0.0);
			float diffuse_weight = (dot(normalize(-lightDir),avgNormal) + 1.0) * 0.5;
			
			//float diffuse_weight = (avgNormal.x + 1.0) * 0.5;
			vec3 E = normalize(worldPos.xyz-camPos);
			//vec3 R = reflect(-lightDir, avgNormal);
			vec3 R = reflect(normalize(vec3(15.0,20.0,10.0) - worldPos.xyz), avgNormal);
			//vec3 R = reflect(-E, avgNormal);
			float specular = pow( max(dot(R, E), 0.0), 
			                 10.0 );
		#else
			float diffuse_weight = 1.0;
			float specular = 0.0;
		#endif
		
		if (drawMode <= 1.0)
		{	//normal drawing mode
			#ifndef EXCLUDE_COLORS
				float red = avgColor.r * (diffuse_weight + specular);
				float green = avgColor.g * (diffuse_weight + specular);
				float blue = avgColor.b * (diffuse_weight + specular);
				//float red = avgColor.r * (diffuse_weight);
				//float green = avgColor.g * (diffuse_weight);
				//float blue = avgColor.b * (diffuse_weight);
				//apply brightness and contrast
				gl_FragColor.r = (0.5 * (1.0 - contrast) + red * brightness * contrast);
				gl_FragColor.g = (0.5 * (1.0 - contrast) + green * brightness * contrast);
				gl_FragColor.b = (0.5 * (1.0 - contrast) + blue * brightness * contrast);
			#else
				gl_FragColor.r = (0.5 * (1.0 - contrast) + (diffuse_weight + specular) * brightness * contrast);
				gl_FragColor.g = gl_FragColor.r;
				gl_FragColor.b = gl_FragColor.r;
			#endif
			gl_FragColor.a = 1.0;
		}
		else if (drawMode == 2.0)
		{	//draw average normals, without lighting
			#ifndef USE_LIGHTING
				vec3 avgNormal = summedNormals.xyz / normalLength;
				if (avgNormal.z < 0)
					avgNormal = -avgNormal;
			#endif
			gl_FragColor.rgb = avgNormal;
			gl_FragColor.a = 1.0;
		} 
		else if (drawMode == 3.0)//(drawMode == 0.0)
		{	//draw depth pass buffer, without lighting
			#ifndef USE_LIGHTING
				float depth = float(texture2D(depthTex,vec2(gl_FragCoord.x/width, gl_FragCoord.y/height)));
				vec4 screenPos = vec4(gl_FragCoord.x/width,gl_FragCoord.y/height, depth, 1.0) * 2.0 - 1.0;
				vec4 worldPos = inverseM * screenPos;
			#endif
			gl_FragColor.r = 1.0 - depth * worldPos.w;
			gl_FragColor.g = gl_FragColor.r;
			gl_FragColor.b = gl_FragColor.r;
			gl_FragColor.a = 1.0;
		}
	}
	//gl_FragColor = vec4(1.0,0.0,0.0,1.0);
}
